searchData={"items":[{"type":"module","title":"Streaming","doc":"`streaming` is a [stream](https://hexdocs.pm/elixir/Stream.html) comprehension macro for Elixir\ninspired by [`for`](https://hexdocs.pm/elixir/Kernel.SpecialForms.html#for/1).\n\n## `streaming` vs `for`\n\n| Feature              | `for`                                | `streaming`            |\n|----------------------|--------------------------------------|------------------------|\n| Evaluation           | Eager                                | Lazy                   |\n| Returns              | Enumerable (or anything if reducing) | Stream                 |\n| Generators           | yes                                  | yes                    |\n| Bitstring generators | yes                                  | yes                    |\n| Filters              | yes                                  | yes                    |\n| `uniq`               | yes                                  | yes                    |\n| `into`               | collection returned                  | collect as side-effect |\n| `reduce`             | yes                                  | n/a - just use `for`   |\n| `scan`               | no                                   | yes                    |\n| `unfold`             | n/a                                  | yes                    |\n| `resource`           | n/a                                  | yes                    |\n| `transform`          | n/a                                  | yes                    |","ref":"Streaming.html"},{"type":"module","title":"Examples - Streaming","doc":"Generate a stream of all permutations of x, y, and z.\n```elixir\nstreaming x <- 1..10, y <- 11..20, z <- 21..30 do\n  {x, y, z}\nend\n```\n\nJust like `for`, mismatching values are rejected.\n```elixir\nusers = [user: \"john\", admin: \"meg\", guest: \"barbara\"]\n\nstreaming {type, name} when type != :guest <- users do\n  String.upcase(name)\nend\n|> Enum.to_list()\n\n=> [\"JOHN\", \"MEG\"]\n```\n\nFilter for keeping only even numbers\n```elixir\nstreaming x <- 1..100, rem(y, 2) == 0 do\n  x\nend\n|> Enum.take(3)\n\n=> [2, 4, 6]\n```\n\n`uniq` works like expected.\n```elixir\nstreaming x <- ~c\"ABBA\", uniq: true do\n  x + 32\nend\n|> Enum.to_list()\n\n=> ~c\"ab\"\n```\n\nInfinite stream of multiples of 2 with `unfold`, based on\n[`Stream.unfold/2`](https://hexdocs.pm/elixir/Stream.html#unfold/2).\n```elixir\nstreaming unfold: 1 do\n  n -> {n, n * 2}\nend\n|> Enum.take(10)\n\n=> [1, 2, 4, 8, 16, 32, 64, 128, 256, 512]\n```\n\nSupport for [`Stream.scan/3`](https://hexdocs.pm/elixir/Stream.html#scan/3) is included\n(but not [`Stream.scan/2`](https://hexdocs.pm/elixir/Stream.html#scan/2))\n```elixir\nstreaming x <- 1..5, scan: 0 do\n  acc -> x + acc\nend\n|> Enum.to_list()\n\n=> [1, 3, 6, 10, 15]\n```\n\nStream values from a resource and close it when the stream ends. The `after` block is required\nwhen using a resource.\n```elixir\nstreaming resource: StringIO.open(\"string\") |> elem(1) do\n  pid ->\n    case IO.getn(pid, \"\", 1) do\n      :eof -> {:halt, pid}\n      char -> {[char], pid}\n    end\nafter\n  pid -> StringIO.close(pid)\nend\n|> Enum.to_list()\n\n=> [\"s\", \"t\", \"r\", \"i\", \"n\", \"g\"]\n```\n\nTransforming data from a resource with an enumerable. The `after` block is optional when\ntransforming.\n```elixir\nstreaming i <- 1..100, transform: StringIO.open(\"string\") |> elem(1) do\n  pid ->\n    case IO.getn(pid, \"\", 1) do\n      :eof -> {:halt, pid}\n      char -> {[{i, char}], pid}\n    end\nafter\n  pid -> StringIO.close(pid)\nend\n|> Enum.to_list()\n\n=> [{1, \"s\"}, {2, \"t\"}, {3, \"r\"}, {4, \"i\"}, {5, \"n\"}, {6, \"g\"}]\n```\n\nBitstring generators are supported.\n```elixir\npixels = <<213, 45, 132, 64, 76, 32, 76, 0, 0, 234, 32, 15, 34>>\n\nstreaming < > do\n  {r, g, b}\nend\n|> Enum.to_list()\n\n=> [{213, 45, 132}, {64, 76, 32}, {76, 0, 0}, {234, 32, 15}]\n```\n\nBitstring generators can be combined with `scan` and `transform`!\n```elixir\nstreaming < >, transform: StringIO.open(\"string\") |> elem(1) do\n  pid ->\n    case IO.getn(pid, \"\", 1) do\n      :eof -> {:halt, pid}\n      char -> {[{char, i}], pid}\n    end\nafter\n  pid -> StringIO.close(pid)\nend\n|> Enum.to_list()\n\n=> [{\"s\", 115}, {\"t\", 116}, {\"r\", 114}, {\"i\", 105}, {\"n\", 110}, {\"g\", 103}]\n```\n\n\nNote that `into` uses [`Stream.into/2`](https://hexdocs.pm/elixir/Stream.html#into/3)\nand streams values into a collectable _as a side-effect_.\n```elixir\n{:ok, io_device} = StringIO.open(\"\")\nio_stream = IO.stream(io_device, :line)\n\nstreaming s <- [\"hello\", \"there\"], into: io_stream do\n  String.capitalize(s)\nend\n|> Enum.to_list()\n\n=> [\"Hello\", \"There\"]","ref":"Streaming.html#module-examples"},{"type":"module","title":"Values has been streamed into io_stream as well - Streaming","doc":"StringIO.contents(io_device)\n\n=> {\"\", \"HelloThere\"}\n```","ref":"Streaming.html#module-values-has-been-streamed-into-io_stream-as-well"},{"type":"macro","title":"Streaming.streaming/2","doc":"","ref":"Streaming.html#streaming/2"},{"type":"macro","title":"Streaming.streaming/3","doc":"","ref":"Streaming.html#streaming/3"},{"type":"macro","title":"Streaming.streaming/4","doc":"","ref":"Streaming.html#streaming/4"},{"type":"macro","title":"Streaming.streaming/5","doc":"","ref":"Streaming.html#streaming/5"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}